; =========================================================================
;   File....... Nokia 6610 gLCD Demo for EPSON Driver
;   Purpose.... Nokia 6610 gLCD EPSON DRIVER SUITE
;   Author..... Westaust55
;   E-mail.....
;   Started.... 14-01-2012
;   Updated.... 
;  ========================================================================
;
;
; -----[ Program Description ]---------------------------------------------
; Demonstration program for using the 128 x 128 colour gLCD from Nokia mobile/cell phones
; Such displays are available from models including the 6100, 6610, 6610i
; This following code is for the gLCD with the EPSON controller chip as distinguished with the green flex connecting circuit 
; it is NOT intended for the PHILIPS controller as distinguished with the brown flex connecting circuit 
; The code uses the 256 colour (8-bit) mode
;
; -----[ Revision History ]------------------------------------------------
;
;A  14-01-2012  First Release - routines included to:
;	- Initialise gLCD module
;	- Clear the display
;	- Organised shutdown
;	- display text in specified colour (routine can handle 5x7 and 7x9 pixel characters)
;	- plot/set a point in specified colour
;	- draw a line between two point in specified colour
;	- draw a filled or unfilled rectangle in specified colour based on two points
;	- draw a circle in specified colour based on centre coordinates and radius
;	- draw an equal sided diamond in specified colour based on centre coordinates and side length
;
; B  15-01-2012 Modified to use an external EEPROM to hold full font sets.
; C  17-01-2012 Added demo code for display of the 8x10 font
;
;
;
#Terminal 9600
#PICAXE 40X1
#No_Data	
#No_Table

;Define the IO pins for outputs to the gLCD

; - - - DIGITAL INPUT PINS  - - -
; none used

; - - - DIGITAL OUTPUT PINS - - -
; below outputs are specific to the Nokia 6610 gLCD module control/data
SYMBOL SCLK    = 7
SYMBOL SDA     = 6
SYMBOL BckLED  = 5
SYMBOL CS      = 4
SYMBOL RES     = 3
;
; - - - ANALOGUE INPUT PINS  - - -
; none used

;
; -----[ General Variables ]-------------------------------------------------------
;
; Define the user/alias names for the program variables
SYMBOL temp		= w0		; = b17:b16  temp word variable used for intermediate calcs in Demo
; variables b2 to b7 inclusive are free for the user to assign


; -----[ Graphic LCD Specific Variables ]-------------------------------------------------------
;
; The following variables are used in the graphic plotting routines
SYMBOL pointer	= b8		; pointer for current char in text strings and other uses - typically in the Main program section
SYMBOL value	= b9		; value to be passed to the gLCD as command or data
SYMBOL colour	= b10		; Foreground colour to plot
SYMBOL bcolour	= b11		; Background colour to plot
SYMBOL base		= b11		; base offset from axis for drawing lines and used in circles + diamonds

SYMBOL x		= b12		; X-Coordinate (0 to 129)
SYMBOL y		= b13		; Y-Coordinate (0 to 129)
SYMBOL index2	= w7		; = b15:b14 used for values > 255 - used in DrawBox subroutine 
SYMBOL   x2 	= b14		; used to second point X-Coord for lines and boxes
SYMBOL   y2		= b15		; used to second point Y-Coord for lines and boxes
SYMBOL x0		= b16		; used for drawing circles and diamonds
SYMBOL y0		= b17		; used for drawing circles and diamonds
SYMBOL char		= b17		; holds the current character value from a string for plotting
SYMBOL slope	= b18		; used for drawing lines
SYMBOL radius	= b18		; used for drawing circles
SYMBOL length	= b18		; used for drawing diamonds
SYMBOL width	= b19		; define the width (x-coord) of plot area for graphic or character
SYMBOL height	= b20		; define the height (y-coord) of plot area for graphic or character
SYMBOL sign       = b21		; used for drawing lines
SYMBOL boxfill 	= b21		; flag used to indicate if box is filled or unfilled
SYMBOL diacirc	= b21		; flag used for drawing circles and diamonds
SYMBOL EEbase	= w11		; holds the starting address for the specified font set for character display
SYMBOL   dx       = b22		; used for drawing lines, circles and diamonds
SYMBOL   dy       = b23		; used for drawing lines, circles and diamonds
SYMBOL EEAddr	= w12		; = b25:b24 = EEPROM address pointer - word variable required for 24LC256 external EEPROM
SYMBOL index	= w13		; = b27:b26 index for pointers/loops requiring values > 255  eg ClearLCD and DrawBox subroutines
SYMBOL   indexlo	= b26		; separately use the lower byte of 'index' when smaller values (0 to 255) suffice for nested loops
SYMBOL   indexhi	= b27		; separately use the high byte of 'index' when smaller values (0 to 255) suffice for nested loops

;
; -----[ Constants ]-------------------------------------------------------
;
; Define names for the EPSON controller chip commands
; note these are provided here for reference but the raw values are used in the routines below
SYMBOL CASET	= $15 	; Column address set
SYMBOL PWRCTR	= $20 	; Power control
SYMBOL NOP		= $25 	; NOP instruction
SYMBOL RAMWR	= $5C 	; Writing to memory
SYMBOL RAMRD	= $5D 	; Reading from memory
SYMBOL PASET	= $75 	; Page address set
SYMBOL EPSRRD1	= $7C 	; Read register 1
SYMBOL EPSRRD2	= $7D 	; Read register 2
SYMBOL VOLCTR	= $81 	; Electronic volume control
SYMBOL TMPGRD	= $82 	; Temperature gradient set
SYMBOL SLPOUT	= $94 	; Sleep out/off
SYMBOL SLPIN	= $95 	; Sleep in/on
SYMBOL DISNOR	= $A6 	; Normal display
SYMBOL DISINV	= $A7 	; Inverse display
SYMBOL PTLIN	= $A8 	; Partial display in
SYMBOL PTLOUT	= $A9 	; Partial display out
SYMBOL ASCSET	= $AA 	; Area scroll set
SYMBOL SCSTART	= $AB 	; Scroll start set
SYMBOL DISOFF	= $AE 	; Display off
SYMBOL DISON	= $AF 	; Display on
SYMBOL COMSCN	= $BB 	; Common scan direction
SYMBOL DATCTL	= $BC 	; Data scan direction, etc.
SYMBOL DISCTL	= $CA 	; Display control
SYMBOL EPCOUT	= $CC 	; Cancel EEPROM control
SYMBOL EPCTIN	= $CD 	; Control EEPROM
SYMBOL RGBSET8	= $CE 	; 256-color position set
SYMBOL OSCON	= $D1 	; Internal oscillation on
SYMBOL OSCOFF	= $D2 	; Internal oscillation off
SYMBOL VOLUP	= $D6 	; Increment contrast control by 1
SYMBOL VOLDOWN	= $D7 	; Decrement contrast control by 1
SYMBOL RMWIN	= $E0 	; Read and modify write
SYMBOL RMWOUT	= $EE 	; End
SYMBOL EPMWR	= $FC 	; Write into EEPROM
SYMBOL EPMRD	= $FD 	; Read from EEPROM

; Define constants for the gLCD controller chip D/C command
SYMBOL LCD_C = $00		; msb = 0 for Command 
SYMBOL LCD_D = $80		; msb = 1 for Data 


; Define constants for some colours in the 256 colour 8-bit RGB format (RRR-GGG-BB)
SYMBOL Black	= $00		; % 000 000 00
SYMBOL Blue		= $03		; % 000 000 11
SYMBOL Green	= $1C		; % 000 111 00
SYMBOL Cyan		= $1F		; % 000 111 11 = Blue + Green
SYMBOL Brown	= $AD  	; % 101 011 01
SYMBOL Pink		= $CE		; % 110 011 10
SYMBOL Red		= $E0		; % 111 000 00
SYMBOL Magenta	= $E3		; % 111 000 11 = Red  + Blue
SYMBOL Orange	= $F0		; % 111 100 00
SYMBOL Yellow	= $FC		; % 111 111 00 = Red  + Green
SYMBOL White	= $FF		; % 111 111 11

; Scratchpad useage
; Identifiy scratchpad locations used for temporary storage
SYMBOL sp_x		= 0	; Used in Drawbox subroutine
SYMBOL sp_y		= 1	; Used in Drawbox subroutine
SYMBOL sp_x2	= 2	; Used in Drawbox subroutine
SYMBOL sp_y2	= 3	; Used in Drawbox subroutine


; EEPROM Slave address and Font Base addresses
SYMBOL eeprom_0 = %10100000
SYMBOL Font6x8 = $0400	; start of the 6x8  font in external EEPROM
SYMBOL Font8x10 =$0900	; start of the 8x10 font in external EEPROM



; -----[ Initialization ]--------------------------------------------------
;
Init:
	SETFREQ m8 
	PAUSE 1000				; give time for the Programming Editor terminal window to open
	GOSUB InitLCD			; initialise the gLCD ready for use
	SERTXD ("Start clearing - backlight is turned ON once cleared", cr, lf)
	colour = Black			; set the initial "background" colour for clearing	
	GOSUB ClearLCD			; clear the LCD to black colour
	SERTXD ("Done clearing gLCD memory", cr, lf)
	HIGH BckLED				; turn on the backlight
	HI2CSETUP i2cMaster, eeprom_0, i2cfast, i2cword ; set up for the external EEPROM holding font data
;
;
; -----[ Main Program Code ]----------------------------------------------------
;
Main:
	; Plot some waveforms in a few different colours
	FOR x = 0 TO 129
 
 		temp = x * 3
		y = SIN temp / 2
		IF y <64 THEN
			y = 50 - y
		ELSE
			y = y - 14
		ENDIF
		colour = Red
		GOSUB GotoXY
		GOSUB Plot
		
		temp = x * 3 + 120
		y = SIN temp / 2
		IF y <64 THEN
			y = 50 - y
		ELSE
			y = y - 14
		ENDIF
		colour = Blue
		GOSUB GotoXY
		GOSUB Plot
		
		temp = x * 3 + 240
		y = SIN temp / 2
		IF y <64 THEN
			y = 50 - y
		ELSE
			y = y - 14
		ENDIF
		colour = White
		GOSUB GotoXY
		GOSUB Plot		
		
		temp = x * 3
		y = COS temp / 2
		IF y <64 THEN
			y = 78 - y
		ELSE
			y = y + 14
		ENDIF
		colour = Green
		GOSUB GotoXY
		GOSUB Plot
		
		temp = x * 3 + 120
		y = COS temp / 2 
		IF y <64 THEN
			y = 78 - y
		ELSE
			y = y + 14
		ENDIF
		colour = Yellow
		GOSUB GotoXY
		GOSUB Plot		
		
		temp = x * 3 + 240
		y = COS temp / 2 
		IF y <64 THEN
			y = 78 - y
		ELSE
			y = y + 14
		ENDIF
		colour = Orange
		GOSUB GotoXY
		GOSUB Plot
	NEXT
	
	; Now let user examine the display while incrementing/looping the contrast through the entire available range.
	; The increment and decrement volume/contrast functions only work with the 'alpha' (first) parameter.
	; For the full electronic volume/contrast control which only uses the lower 6 bits, the range is
	; $00 to $3F and the actual value wraps around after $3F to $00.

	PAUSE 5000			; wait a while so user can see what has happened this far . . . .
  	
	SERTXD ("Now changing contrast", cr, lf)
	FOR pointer = $00 TO $3F
		value = $D6
		GOSUB SendCmd		 ; Command to increment Contrast by 1 over the full 6-bit range
		  SERTXD (#Pointer, " ") ; Indicate increment number to enable user to consider if need to adjust InitLCD setting
		PAUSE 500
	NEXT pointer
  	
  	PAUSE 5000 ; wait so we can see what has happens this  far
	colour = Black			; set the initial "background" colour for clearing
  	GOSUB ClearLCD

	; send a character string to the gLCD
	colour  = red		; define the test colour
	bcolour = black		; define the background colour
  	X = 0 : Y = 0 		; define the top-left start x,y 
	GOSUB  Font6x8AtXY		; go to the start point for the test string		
	FOR pointer = 0 TO 21
		LOOKUP pointer,("# HELLO PICAXE FORUM #"),char
		GOSUB SendChar	; send data to the gLCD
	NEXT pointer 
	
	; follow with a second character string to the gLCD
	colour  = white		; define the test colour
	bcolour = blue		; define the background colour
  	X = 0 : Y = 16		; define the top-left start x,y 
	GOSUB  Font6x8AtXY	; go to the start point for the test string		
	FOR pointer = 0 TO 20
		LOOKUP pointer,("## From WestAust55 ##"),char
		GOSUB SendChar	; send data to the gLCD
	NEXT pointer 
	
;		; follow with a third character string to the gLCD
;	colour  = red		; define the test colour
;	bcolour = black		; define the background colour
;  	X = 16 : Y = 30		; define the top-left start x,y 
;	GOSUB  Font8x10AtXY	; go to the start point for the test string		
;	FOR pointer = 0 TO 1
;		LOOKUP pointer,("01"),char
;		GOSUB SendChar	; send data to the gLCD
;	NEXT pointer 
	
	; draw some lines
	x = 88 : y = 35
	x2 = 125 : y2 = 90
	Colour = Yellow
	GOSUB DrawLine
	
	x = 5 : y = 40
	x2 = 90 : y2 = 90
	Colour = Cyan
	GOSUB DrawLine

	; draw filled and then as a border an unfilled box
	x = 10 : y = 75
	x2 =60 : y2 = 125
	boxfill = 1
	Colour = Orange
	GOSUB DrawBox

	x = 10 : y = 75
	x2 = 60 : y2 = 125
	boxfill = 0
	Colour = Green
	GOSUB DrawBox	
	
	; draw some concentric circles
	x =64 : y = 64 : radius = 20 
	Colour = Blue
	GOSUB DrawCircle
	
	x =64 : y = 64 : radius = 40 
	Colour = Pink
	GOSUB DrawCircle
 
 	;draw a diamond
 	x =105 : y = 105 : length = 30 
	Colour = White
	GOSUB DrawDiamond


   	PAUSE 50000				; delay to give user time to see the results
  	colour = Black			; set the initial "background" colour for clearing	
	GOSUB ClearLCD			; clear the LCD to black colour 

 
 	; now we display the entire 6x8 font set
 
 	; send a character string to the gLCD
	colour  = red		; define the test colour
	bcolour = black		; define the background colour
  	X = 0 : Y = 0 		; define the top-left start x,y 
	GOSUB  Font6x8AtXY		; go to the start point for the test string		
	FOR pointer = 0 TO 20
		LOOKUP pointer,(" !", 34,"#$%&'()*+,-./01234"),char
		GOSUB SendChar	; send data to the gLCD
	NEXT pointer 
 
 	; send a character string to the gLCD
	colour  = red		; define the test colour
	bcolour = black		; define the background colour
  	X = 0 : Y = 9 		; define the top-left start x,y 
	GOSUB  Font6x8AtXY		; go to the start point for the test string		
	FOR pointer = 0 TO 20
		LOOKUP pointer,("56789:;<=>?@ABCDEFGHI"),char
		GOSUB SendChar	; send data to the gLCD
	NEXT pointer 
	
		; send a character string to the gLCD
	colour  = red		; define the test colour
	bcolour = black		; define the background colour
  	X = 0 : Y = 18 		; define the top-left start x,y 
	GOSUB  Font6x8AtXY		; go to the start point for the test string		
	FOR pointer = 0 TO 20
		LOOKUP pointer,("JKLMNOPQRSTUVWXYZ[",92,"]^"),char
		GOSUB SendChar	; send data to the gLCD
	NEXT pointer 
	
	
		; send a character string to the gLCD
	colour  = red		; define the test colour
	bcolour = black		; define the background colour
  	X = 0 : Y = 27 		; define the top-left start x,y 
	GOSUB  Font6x8AtXY		; go to the start point for the test string		
	FOR pointer = 0 TO 20
		LOOKUP pointer,("_`abcdefghijklmnopqrs"),char
		GOSUB SendChar	; send data to the gLCD
	NEXT pointer 
	
	
		; send a character string to the gLCD
	colour  = red		; define the test colour
	bcolour = black		; define the background colour
  	X = 0 : Y = 36 		; define the top-left start x,y 
	GOSUB  Font6x8AtXY		; go to the start point for the test string		
	FOR pointer = 0 TO 20
		LOOKUP pointer,("tuvwxyz{|}~          "),char
		GOSUB SendChar	; send data to the gLCD
	NEXT pointer 
 
 	colour  = red		; define the test colour
	bcolour = black		; define the background colour
  	X = 0 : Y = 45 		; define the top-left start x,y 
	GOSUB  Font6x8AtXY
 	FOR pointer = $80 to $8F
 		char = pointer
 		GOSUB SendChar
 	NEXT pointer
 	
 	colour  = red		; define the test colour
	bcolour = black		; define the background colour
  	X = 0 : Y = 54 		; define the top-left start x,y 
	GOSUB  Font6x8AtXY
 	FOR pointer = $90 to $9F
 		char = pointer
 		GOSUB SendChar
 	NEXT pointer
 	
 	colour  = red		; define the test colour
	bcolour = black		; define the background colour
  	X = 0 : Y = 63 		; define the top-left start x,y 
	GOSUB  Font6x8AtXY
 	FOR pointer = $A0 to $AF
 		char = pointer
 		GOSUB SendChar
 	NEXT pointer
 	
 	colour  = red		; define the test colour
	bcolour = black		; define the background colour
  	X = 0 : Y = 72 		; define the top-left start x,y 
	GOSUB  Font6x8AtXY
 	FOR pointer = $B0 to $BF
 		char = pointer
 		GOSUB SendChar
 	NEXT pointer
 	
    	PAUSE 50000				; delay to give user time to see the results
  	colour = Black			; set the initial "background" colour for clearing	
	GOSUB ClearLCD			; clear the LCD to black colour
 
 
  	; now we display the entire 8X10 font set
 
 	; send a character string to the gLCD
	colour  = red		; define the test colour
	bcolour = black		; define the background colour
  	X = 0 : Y = 0 		; define the top-left start x,y 
	GOSUB  Font8X10AtXY		; go to the start point for the test string		
	FOR pointer = 0 TO 15
		LOOKUP pointer,(" !", 34,"#$%&'()*+,-./"),char
		GOSUB SendChar	; send data to the gLCD
	NEXT pointer 
 
 	; send a character string to the gLCD
	colour  = red		; define the test colour
	bcolour = black		; define the background colour
  	X = 0 : Y = 11 		; define the top-left start x,y 
	GOSUB  Font8X10AtXY		; go to the start point for the test string		
	FOR pointer = 0 TO 15
		LOOKUP pointer,("0123456789:;<=>?"),char
		GOSUB SendChar	; send data to the gLCD
	NEXT pointer 
	
		; send a character string to the gLCD
	colour  = red		; define the test colour
	bcolour = black		; define the background colour
  	X = 0 : Y = 22 		; define the top-left start x,y 
	GOSUB  Font8X10AtXY		; go to the start point for the test string		
	FOR pointer = 0 TO 15
		LOOKUP pointer,("@ABCDEFGHIJKLMNO"),char
		GOSUB SendChar	; send data to the gLCD
	NEXT pointer 
	
	
		; send a character string to the gLCD
	colour  = red		; define the test colour
	bcolour = black		; define the background colour
  	X = 0 : Y = 33 		; define the top-left start x,y 
	GOSUB  Font8X10AtXY		; go to the start point for the test string		
	FOR pointer = 0 TO 15
		LOOKUP pointer,("PQRSTUVWXYZ[",92,"]^_"),char
		GOSUB SendChar	; send data to the gLCD
	NEXT pointer 
	
	; send a character string to the gLCD
	colour  = red		; define the test colour
	bcolour = black		; define the background colour
  	X = 0 : Y = 44 		; define the top-left start x,y 
	GOSUB  Font8X10AtXY		; go to the start point for the test string		
	FOR pointer = 0 TO 15
		LOOKUP pointer,("`abcdefghijklmno"),char
		GOSUB SendChar	; send data to the gLCD
	NEXT pointer 
	
	
		; send a character string to the gLCD
	colour  = red		; define the test colour
	bcolour = black		; define the background colour
  	X = 0 : Y = 55 		; define the top-left start x,y 
	GOSUB  Font8X10AtXY		; go to the start point for the test string		
	FOR pointer = 0 TO 15
		LOOKUP pointer,("pqrstuvwxyz{|}~",$7F),char
		GOSUB SendChar	; send data to the gLCD
	NEXT pointer 
 
 	colour  = red		; define the test colour
	bcolour = black		; define the background colour
  	X = 0 : Y = 66 		; define the top-left start x,y 
	GOSUB  Font8X10AtXY
 	FOR pointer = $80 to $8F
 		char = pointer
 		GOSUB SendChar
 	NEXT pointer
 	
 	colour  = red		; define the test colour
	bcolour = black		; define the background colour
  	X = 0 : Y = 77 		; define the top-left start x,y 
	GOSUB  Font8X10AtXY
 	FOR pointer = $90 to $9F
 		char = pointer
 		GOSUB SendChar
 	NEXT pointer
 	
 	colour  = red		; define the test colour
	bcolour = black		; define the background colour
  	X = 0 : Y = 88 		; define the top-left start x,y 
	GOSUB  Font8X10AtXY
 	FOR pointer = $A0 to $AF
 		char = pointer
 		GOSUB SendChar
 	NEXT pointer
 	
 	colour  = red		; define the test colour
	bcolour = black		; define the background colour
  	X = 0 : Y = 99 		; define the top-left start x,y 
	GOSUB  Font8X10AtXY
 	FOR pointer = $B0 to $BF
 		char = pointer
 		GOSUB SendChar
 	NEXT pointer
 
 
  	SERTXD (cr, lf, "All done", cr, lf)
  	PAUSE 65000		; delay to give user time to see the results 
  	SERTXD (cr, lf, "Shutting down the gLCD", cr, lf) 	
  	GOSUB Shutdown
  	
	DO
	LOOP	; loop forever
	END

;
; -----[ Subroutines ]-----------------------------------------------------
;
;---------------------------------------
; Routine to initialise the gLCD with EPSON driver chip
InitLCD:
; perform signal state initialisation and gLCD reset
	HIGH CS
	PAUSE 10
	HIGH SCLK
	LOW SDA
	LOW RES
	PAUSE 1
	HIGH RES
	PAUSE 20
	LOW CS

	; turn on the display and configure the gLCD for 8-colour mode
	value = $CA : GOSUB SendCmd	; Display Control Command (takes 3 byte parameters)  
						; MUST be set before Sleepout and not adjusted when display is turned on.
	value = $00 : GOSUB SendData	; lower 4 bits used : CL diving ratio, F1 and F2 drive pattern
	value = $20 : GOSUB SendData	; lower 6 bits used : Drive duty
	value = $12 : GOSUB SendData	; by datasheet, bit4 MUST be 1, lower 4 bits used - FR inverse-set value

	value = $BB : GOSUB SendCmd	; Common Scan Direction Command (takes 1 byte parameter)  
	value = $01 : GOSUB SendData	; lower 3 bits used : $00 = 1->80 & 81->160;  $01 = 1->80 & 160->81
						; need for value $01 suggests Nokia reverse connections for COM81 pin to COM 160 pin

	value = $D1 : GOSUB SendCmd	; Internal Oscillator On command (takes 0 byte parameters)

	value = $94 : GOSUB SendCmd	; Sleepout command - brings the gLCD out of Sleep mode (takes 0 byte parameters)

	value = $81 : GOSUB SendCmd	; Electronic Volume (Contrast) Control Command (takes 2 byte parameters) 
	value = $20 : GOSUB SendData	; lower 6 bits used : V1 volume/contrast value 'alpha' $00 to $3F is valid
	value = $01 : GOSUB SendData	; lower 3 bits used : 1+ Rb/Ra = specifies internal resistance ratio


	value = $20 : GOSUB SendCmd	; Power Control Command (takes 1 byte parameter)
	value = $0F : GOSUB SendData	; lower 4 bits used :  b3=prim boost circuit; b2=secondary boost circuit;
						; for each 1 = 'ON'    b1=voltage regulator;  b0=Reference voltage generator. 

	PAUSE 200				; wait a while for power circuits to initialise

	value = $A7 : GOSUB SendCmd	; Inverse Display command (takes 0 byte parameters)


	value = $BC : GOSUB SendCmd	; Data Control Command (takes 3 byte parameters)
	value = $00 : GOSUB SendData	; lower 3 bits used : Normal/Inverse page address and scan direction
	value = $00 : GOSUB SendData	; lower 3 bits used : RGB arrangement %000=RGBRGB...RGB; $001=BGRBGR...BGR
	value = $01 : GOSUB SendData	; lower 3 bits used : Gray-scale setup $01=8 gray-scale; $02 =16 gray-scale

	value = $CE : GOSUB SendCmd	; 256-colour Position Set Command (takes 20 byte parameters)
						; For each of the following 20 bytes only the lower 4 bits are used.
	value = $00 : GOSUB SendData	; Intermediate Red   tone 000
	value = $02 : GOSUB SendData	; Intermediate Red   tone 001
	value = $04 : GOSUB SendData	; Intermediate Red   tone 010
	value = $06 : GOSUB SendData	; Intermediate Red   tone 011
	value = $08 : GOSUB SendData	; Intermediate Red   tone 100
	value = $0A : GOSUB SendData	; Intermediate Red   tone 101
	value = $0C : GOSUB SendData	; Intermediate Red   tone 110
	value = $0F : GOSUB SendData	; Intermediate Red   tone 111
	
	value = $00 : GOSUB SendData	; Intermediate Green tone 000
	value = $02 : GOSUB SendData	; Intermediate Green tone 001
	value = $04 : GOSUB SendData	; Intermediate Green tone 010
	value = $06 : GOSUB SendData	; Intermediate Green tone 011
	value = $08 : GOSUB SendData	; Intermediate Green tone 100
	value = $0A : GOSUB SendData	; Intermediate Green tone 101
	value = $0C : GOSUB SendData	; Intermediate Green tone 110
	value = $0F : GOSUB SendData	; Intermediate Green tone 111

	value = $00 : GOSUB SendData	; Intermediate Blue  tone  00
	value = $05 : GOSUB SendData	; Intermediate Blue  tone  01
	value = $0A : GOSUB SendData	; Intermediate Blue  tone  10
	value = $0F : GOSUB SendData	; Intermediate Blue  tone  11

	value = $25 : GOSUB SendCmd	; NOP Instruction (takes 0 byte parameters)
	value = $AF : GOSUB SendCmd	; Display ON command (takes 0 byte parameters)

	RETURN
;---------------------------------------
; Routine to shut down the EPSON controller and gLCD in an organised manner
Shutdown:
	value = $AE : GOSUB SendCmd	; Display OFF command (takes 0 byte parameters)
	value = $95 : GOSUB SendCmd	; Enter Sleep mode command (takes 0 byte parameters)
	LOW CS				; Chip Select OFF
	LOW BckLED				; Turn the gLCD Backlight OFF
	RETURN

;---------------------------------------
; Subroutine to rapidly clear the gLCD with a colour defined before calling
ClearLCD:
	x = 0	: width  = 130		; set the "cursor" position to the screen origin for sequential clear
	y = 0 : height = 130  		; and the width/height for full screen
	GOSUB GotoXY			; move "cursor" to the desired start point
	value = $5C : GOSUB SendCmd	; Send the command to start writing to LCD RAM
	value = colour			; set the colour to be used for clearing screen
	FOR index = 0 TO 3380 ; 16900		; step through all 130 x 130 gLCD module visible pixel locations
		SHIFTOUT SCLK, SDA, MSBFirst_L, (LCD_D/1, value/8, LCD_D/1, value/8, LCD_D/1, value/8, LCD_D/1, value/8, LCD_D/1, value/8)
	NEXT
	RETURN
	
;---------------------------------------
; Calling to this point when moving to a point x,y ready to display a character will automatically set the character width and height
Font8x10AtXY:
	width  = 8				; font chars occupy 6 actual pixels in x-coord (5 for char + 1 for inter char space)
	height = 10				; font chars occupy 8 actual pixels in y-coord (7 for char + 1 for inter line space)
	EEbase = Font8x10			; define the font set starting address
	GOTO GotoXY
;---------------------------------------
; Calling to this point when moving to a point x,y ready to display a character will automatically set the character width and height
Font6x8AtXY:
	width  = 6				; font chars occupy 6 actual pixels in x-coord (5 for char + 1 for inter char space)
	height = 8				; font chars occupy 8 actual pixels in y-coord (7 for char + 1 for inter line space)
	EEbase = Font6x8			; define the font set starting address
;	GOTO GotoXY				; not required as this drops through into the 'normal' GotoXY routine immediately below
;---------------------------------------
; Subroutine to set the start x,y and end x,y for the defined/desired screen area
GotoXY:
	value = $15		: GOSUB SendCmd	; Column Address Set command (takes 2 byte parameters)
	value = x		: GOSUB SendData	; Start Address (must be lower than the 'End Address' value)
	value = x+width-1	: GOSUB SendData	; End Address
	
	value = $75		: GOSUB SendCmd	; Page Address Set command (takes 2 byte parameters)
	value = y + 2	: GOSUB SendData	; Start page (must be lower than the 'End Page' value)
	value = y+height+1 : GOSUB SendData	; End Page  (note value is in effect y + height + 2 - 1) 
	RETURN
;---------------------------------------
; Subroutine to set the colour for a pixel at an already established x,y co-ordinate
Plot:
	value = $5C	: GOSUB SendCmd		; Send command for Writing to LCD RAM memory command
	value = colour	: GOSUB SendData	; send the required colour (256 colour 8-bit mode as RRR-GGG-BB)
	RETURN
;---------------------------------------
; Subroutine to send a command to the gLCD module
SendCmd:
	SHIFTOUT SCLK, SDA, MSBFirst_L, (LCD_C/1, value/8)
	RETURN
;---------------------------------------
; Subroutine to send a data value to the gLCD module
SendData:
	SHIFTOUT SCLK, SDA, MSBFirst_L, (LCD_D/1, value/8)
	RETURN
;---------------------------------------
; Subroutine to plot one character to the gLCD at location x,y - font size is defined by calling 'FontAtXY' before calling this routine
SendChar:

; demo version here - use if using the internal EEPROM with limited character set
#REM
	IF char = "0" or char = "1" THEN	; this test is used to take the chars 0 and 1 for the 7x9 font example
	       char = char - $30
	ELSEIF char = $20 THEN			 ; Detect: space
		char = $03
	ELSEIF char = $23 OR char = $24 THEN ; Detect: # and $
		char = char - $1F 	 		
	ELSEIF char > $40 AND char < $5B THEN ; Detect: uppercase A to Z	
		char = char - $3B
	ELSEIF char > $60 AND char < $7B THEN ; Detect: lowercase a to z but use uppercase	
		char = char - $5B
	ELSE
		char = $03 ; $00			; we do not have EEPROM space so just print a space for all others
	ENDIF
#endrem

; This is the nomal version for just a 5x7 font
; revert to this when the First two larger chars in EEPROM are removed

	char = char - $20
	EEAddr = char * height + EEbase	; pointer to EEPROM location for first byte of font character using 1 bytes per char row
	value = $5C : GOSUB SendCmd		; Send command ready for Writing to LCD RAM memory command

	FOR indexhi = 1 to height
		HI2CIN EEAddr, (char)
		FOR indexlo = 1 TO width	; we only use 6 bits from each byte as 5 wide for char and 1 blank as separator
			IF char BIT 7 SET THEN	; if the current msb is 1 then this is part of the character
				value = colour	; so we use the text colour
			ELSE
				value = bcolour	; otherwise we select the nominated background colour
			ENDIF
			GOSUB SendData 		; write colour data for the current pixel in character area to the gLCD RAM
			char = char << 1		; shift byte left ready to check next pixel
		NEXT indexlo			; loop until all pixels in current row done for this character
		INC EEAddr				; increment EEPROM address pointer for data for next row of current character
	NEXT indexhi				; loop until all rows are plotted for the current character
	x = x + width				; advance the x-coord pointer ready for the next character
	GOSUB GotoXY				; advance the working area right one character area ready for the next character
	RETURN
;---------------------------------------
; Subroutine to draw a line between two point defined with x,y as first point and width,height as the second point (x2,y2)

DrawLine:
	width = 1 : height = 1			; each point is a single pixel
	If x2 < x THEN				; If first point is to the right of second point then swap so we draw left to right
		SWAP x, x2
		SWAP y, y2
	ENDIF		
	
	IF y2 < y THEN
      	sign = 1 				; negative slope
		dy = y - y2
	ELSE
		sign = 0 				; positive slope
		dy = y2 - y  
	ENDIF
      dx = x2 - x
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -        
	IF dy > dx THEN           		; test for steep slope lines
          IF y2 < y THEN         		; continue here for steep sloped lines handled different to shallow lines
		SWAP x, x2
		SWAP y, y2
          ENDIF
	    
	    IF x2 < x THEN
            sign = 1 				; negative slope
            dx = x - x2
          ELSE
            sign = 0 				; positive slope
            dx = x2 - x  
          ENDIF
          
          dy = y2 - y
          slope = 100 * dx / dy          	; multiplier of 100 used for accuracy to 2 decimal place 
          base = x
          dy = y
          FOR indexlo = dy to y2          ; for each pixel in the y axis  (NOTE: origin is at top left)    
            x = indexlo - dy * slope / 100
            IF sign = 0 THEN
              x = base + x
            ELSE
              x = base - x  
            ENDIF
            y = indexlo
            GOSUB GotoXY         		; move to required gLCD location
            GOSUB Plot    			; turn on the current pixel
          NEXT indexlo
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -                
        ELSE  					; continue here if it is a shallow slope line
          slope = 100 * dy / dx           ; multiplier of 100 used for accuracy to 2 decimal place
          base  =  y
          dx = x
          FOR indexlo = dx TO x2          ; for each pixel in the x axis  (NOTE: origin is at top left)  
            y = indexlo - dx * slope /100
            IF sign = 0 THEN  
              y = base + y
            ELSE
              y = base - y
            ENDIF
            x=indexlo
            GOSUB GotoXY                    ; move to required gLCD location
            GOSUB Plot                   ; turn on the current pixel
          NEXT indexlo
        ENDIF                              
        RETURN
;---------------------------------------
; subrotine to draw a rectangle
; Enter with x,y and x2,y2, plus the fill requirement and colour specified
DrawBox:
	IF x2 < x THEN	; make sure the first point is on the left
		SWAP x, x2
		SWAP y, y2
	ENDIF
	
	IF y2 < y THEN	; make sure the first point is on the left
		SWAP y, y2
	ENDIF

	IF boxfill = 1 THEN

		width = x2 - x + 1
		height = y2 - y + 1
		GOSUB GOTOXY
		GOSUB Plot				; open the channel to write colour data to RAM
		index2 = width * height		; number of loops
		indexlo = index2// 5
		index2 = index2 / 5
		IF indexlo >0 then 
			INC index2
		ENDIF
		FOR index = 1 to index2 	; For each coloumn and for each row in lots of 5 pixels
		   SHIFTOUT SCLK, SDA, MSBFirst_L, (LCD_D/1, value/8, LCD_D/1, value/8, LCD_D/1, value/8, LCD_D/1, value/8, LCD_D/1, value/8)
		NEXT index
		
	ELSE
	; here for an unfilled box	
	; first must save x,y and x2,y2 to scratchpad to prevent destruction while drawing box
		PUT sp_x, x
		PUT sp_y, y
		PUT sp_x2, x2
		PUT sp_y2, y2
		; draw side 1
		GET sp_x, x
		GET sp_y, y
		GET sp_x2, x2
		GET sp_y, y2
		GOSUB DrawLine
		; draw side 2
		GET sp_x, x
		GET sp_y, y
		GET sp_x, x2
		GET sp_y2, y2
		GOSUB DrawLine
		; draw side 3
		GET sp_x2, x
		GET sp_y, y
		GET sp_x2, x2
		GET sp_y2, y2
		GOSUB DrawLine
		; draw side 4
		GET sp_x, x
		GET sp_y2, y
		GET sp_x2, x2
		GET sp_y2, y2
		GOSUB DrawLine
	ENDIF
	RETURN	
;---------------------------------------
; Entry Point to draw a equal sided diamond centred on position x,y and a sides of specified length in pixels
DrawDiamond:
	diacirc = 1
	radius = length * 100 + 12 / 141
	dy = radius * 2
	GOTO DiaCircCom

;---------------------------------------
DrawCircle:
; Entry Point to draw a circle centred on position x,y and a radius specified in pixels
	diacirc = 0
	dy = 0 - radius - radius

DiaCircCom:
	width = 1 	: height = 1
	x0 = x 	: y0 = y
 	dx = 0	: x2 = 0
	y2 = radius
	base = 1 - radius 
	
	x = x0
	y = y0 + radius	
	GOSUB GOTOXY
	GOSUB Plot
	x = x0 
	y = y0 - radius	
	GOSUB GOTOXY
	GOSUB Plot
	x = x0 + radius
	y = y0	
	GOSUB GOTOXY
	GOSUB Plot
	x = x0 - radius
	y = y0 	
	GOSUB GOTOXY
	GOSUB Plot

	DO WHILE x2 < y2
		IF diacirc = 0 AND base  > 127 THEN GOTO DiaCircCont ; skip next few lines if circle and No in effect is negative
		DEC y2
		dy = dy +2
		base = base + dy
DiaCircCont:
		INC x2
		dx = dx + 2
		base = base  + dx + 1
	
		x = x0 + x2
		y = y0 + y2	
		GOSUB GOTOXY
		GOSUB Plot
		x = x0 - x2
		y = y0 + y2	
		GOSUB GOTOXY
		GOSUB Plot
		x = x0 + x2
		y = y0 - y2	
		GOSUB GOTOXY
		GOSUB Plot		
		x = x0 - x2
		y = y0 - y2	
		GOSUB GOTOXY
		GOSUB Plot
		x = x0 + y2
		y = y0 + x2	
		GOSUB GOTOXY
		GOSUB Plot
		x = x0 - y2
		y = y0 + x2	
		GOSUB GOTOXY
		GOSUB Plot
		x = x0 + y2
		y = y0 - x2	
		GOSUB GOTOXY
		GOSUB Plot		
		x = x0 - y2
		y = y0 - x2	
		GOSUB GOTOXY
		GOSUB Plot
			
	LOOP	
	RETURN
;
;
; -----[ Interrupt Routine (NOT USED HERE) ]-------------------------------------
;
;Interrupt: 
;
;       RETURN
; =================================================
;      THE END
; =================================================

; -----[ EEPROM Data ]-----------------------------------------------------
;
; HERE is our 32 Character 6x8 font data - Actual chars are typically 5x7 pixel with 1 pixel space at right and bottom.	
;
; This is not in fact used as the program reads the data from an external EEPROM for the font set
;
EEPROM    ($00,$00,$00,$00,$00,$00,$00,$00)  ; space  = $20
EEPROM    ($20,$20,$20,$20,$20,$00,$20,$00)  ; ! 
EEPROM    ($50,$50,$50,$00,$00,$00,$00,$00)  ; " 

EEPROM    ($50,$50,$F8,$50,$F8,$50,$50,$00)  ; #
EEPROM    ($20,$78,$A0,$70,$28,$F0,$20,$00)  ; $
EEPROM    ($C0,$C8,$10,$20,$40,$98,$18,$00)  ; %

#REM    ;  insufficient space in EEPROM so ignore some characters
EEPROM    ($40,$A0,$A0,$40,$A8,$90,$68,$00)  ; &
EEPROM    ($30,$30,$20,$40,$00,$00,$00,$00)  ; '
EEPROM    ($10,$20,$40,$40,$40,$20,$10,$00)  ; (
EEPROM    ($40,$20,$10,$10,$10,$20,$40,$00)  ; )
EEPROM    ($00,$20,$A8,$70,$70,$A8,$20,$00)  ; *
EEPROM    ($00,$20,$20,$F8,$20,$20,$00,$00)  ; +
EEPROM    ($00,$00,$00,$00,$30,$30,$20,$40)  ; ,
EEPROM    ($00,$00,$00,$F8,$00,$00,$00,$00)  ; -
EEPROM    ($00,$00,$00,$00,$00,$30,$30,$00)  ; .
EEPROM    ($00,$08,$10,$20,$40,$80,$00,$00)  ; /
EEPROM    ($70,$88,$88,$A8,$88,$88,$70,$00)  ; 0  = $30
EEPROM    ($20,$60,$20,$20,$20,$20,$70,$00)  ; 1
EEPROM    ($70,$88,$08,$70,$80,$80,$F8,$00)  ; 2
EEPROM    ($F8,$08,$10,$30,$08,$88,$70,$00)  ; 3
EEPROM    ($10,$30,$50,$90,$F8,$10,$10,$00)  ; 4
EEPROM    ($F8,$80,$F0,$08,$08,$88,$70,$00)  ; 5
EEPROM    ($38,$40,$80,$F0,$88,$88,$70,$00)  ; 6
EEPROM    ($F8,$08,$08,$10,$20,$40,$80,$00)  ; 7
EEPROM    ($70,$88,$88,$70,$88,$88,$70,$00)  ; 8
EEPROM    ($70,$88,$88,$78,$08,$10,$E0,$00)  ; 9
EEPROM    ($00,$00,$20,$00,$20,$00,$00,$00)  ; :
EEPROM    ($00,$00,$20,$00,$20,$20,$40,$00)  ; ;
EEPROM    ($08,$10,$20,$40,$20,$10,$08,$00)  ; <
EEPROM    ($00,$00,$F8,$00,$F8,$00,$00,$00)  ; =
EEPROM    ($40,$20,$10,$08,$10,$20,$40,$00)  ; >
EEPROM    ($70,$88,$08,$30,$20,$00,$20,$00)  ; ?
EEPROM    ($70,$88,$A8,$B8,$B0,$80,$78,$00)  ; @  = $40
#endrem


EEPROM    ($20,$50,$88,$88,$F8,$88,$88,$00)  ; A
EEPROM    ($F0,$88,$88,$F0,$88,$88,$F0,$00)  ; B
EEPROM    ($70,$88,$80,$80,$80,$88,$70,$00)  ; C
EEPROM    ($F0,$88,$88,$88,$88,$88,$F0,$00)  ; D
EEPROM    ($F8,$80,$80,$F0,$80,$80,$F8,$00)  ; E
EEPROM    ($F8,$80,$80,$F0,$80,$80,$80,$00)  ; F
EEPROM    ($78,$88,$80,$80,$98,$88,$78,$00)  ; G
EEPROM    ($88,$88,$88,$F8,$88,$88,$88,$00)  ; H
EEPROM    ($70,$20,$20,$20,$20,$20,$70,$00)  ; I
EEPROM    ($38,$10,$10,$10,$10,$90,$60,$00)  ; J
EEPROM    ($88,$90,$A0,$C0,$A0,$90,$88,$00)  ; K
EEPROM    ($80,$80,$80,$80,$80,$80,$F8,$00)  ; L
EEPROM    ($88,$D8,$A8,$A8,$A8,$88,$88,$00)  ; M
EEPROM    ($88,$88,$C8,$A8,$98,$88,$88,$00)  ; N
EEPROM    ($70,$88,$88,$88,$88,$88,$70,$00)  ; O
EEPROM    ($F0,$88,$88,$F0,$80,$80,$80,$00)  ; P  = $50
EEPROM    ($70,$88,$88,$88,$A8,$90,$68,$00)  ; Q
EEPROM    ($F0,$88,$88,$F0,$A0,$90,$88,$00)  ; R
EEPROM    ($70,$88,$80,$70,$08,$88,$70,$00)  ; S
EEPROM    ($F8,$A8,$20,$20,$20,$20,$20,$00)  ; T
EEPROM    ($88,$88,$88,$88,$88,$88,$70,$00)  ; U
EEPROM    ($88,$88,$88,$88,$88,$50,$20,$00)  ; V
EEPROM    ($88,$88,$88,$A8,$A8,$A8,$50,$00)  ; W
EEPROM    ($88,$88,$50,$20,$50,$88,$88,$00)  ; X
EEPROM    ($88,$88,$50,$20,$20,$20,$20,$00)  ; Y
EEPROM    ($F8,$08,$10,$70,$40,$80,$F8,$00)  ; Z


#REM    ;  insufficient space in EEPROM so ignore some characters
EEPROM    ($78,$40,$40,$40,$40,$40,$78,$00)  ; [
EEPROM    ($00,$80,$40,$20,$10,$08,$00,$00)  ; \
EEPROM    ($78,$08,$08,$08,$08,$08,$78,$00)  ; ]
EEPROM    ($20,$50,$88,$00,$00,$00,$00,$00)  ; ^
EEPROM    ($00,$00,$00,$00,$00,$00,$F8,$00)  ; _
EEPROM    ($60,$60,$20,$10,$00,$00,$00,$00)  ; `  = $60
EEPROM    ($00,$00,$60,$10,$70,$90,$78,$00)  ; a
EEPROM    ($80,$80,$B0,$C8,$88,$C8,$B0,$00)  ; b
EEPROM    ($00,$00,$70,$88,$80,$88,$70,$00)  ; c
EEPROM    ($08,$08,$68,$98,$88,$98,$68,$00)  ; d
EEPROM    ($00,$00,$70,$88,$F8,$80,$70,$00)  ; e
EEPROM    ($10,$28,$20,$70,$20,$20,$20,$00)  ; f
EEPROM    ($00,$00,$70,$98,$98,$68,$08,$70)  ; g
EEPROM    ($80,$80,$B0,$C8,$88,$88,$88,$00)  ; h
EEPROM    ($20,$00,$60,$20,$20,$20,$70,$00)  ; i
EEPROM    ($10,$00,$10,$10,$10,$90,$60,$00)  ; j
EEPROM    ($80,$80,$90,$A0,$C0,$A0,$90,$00)  ; k
EEPROM    ($60,$20,$20,$20,$20,$20,$70,$00)  ; l
EEPROM    ($00,$00,$D0,$A8,$A8,$A8,$A8,$00)  ; m
EEPROM    ($00,$00,$B0,$C8,$88,$88,$88,$00)  ; n
EEPROM    ($00,$00,$70,$88,$88,$88,$70,$00)  ; o
EEPROM    ($00,$00,$B0,$C8,$C8,$B0,$80,$80)  ; p  = $70
EEPROM    ($00,$00,$68,$98,$98,$68,$08,$08)  ; q
EEPROM    ($00,$00,$B0,$C8,$80,$80,$80,$00)  ; r
EEPROM    ($00,$00,$78,$80,$70,$08,$F0,$00)  ; s
EEPROM    ($20,$20,$F8,$20,$20,$28,$10,$00)  ; t
EEPROM    ($00,$00,$88,$88,$88,$98,$68,$00)  ; u
EEPROM    ($00,$00,$88,$88,$88,$50,$20,$00)  ; v
EEPROM    ($00,$00,$88,$88,$A8,$A8,$50,$00)  ; w
EEPROM    ($00,$00,$88,$50,$20,$50,$88,$00)  ; x
EEPROM    ($00,$00,$88,$88,$78,$08,$88,$70)  ; y
EEPROM    ($00,$00,$F8,$10,$20,$40,$F8,$00)  ; z
EEPROM    ($10,$20,$20,$40,$20,$20,$10,$00)  ; {
EEPROM    ($20,$20,$20,$00,$20,$20,$20,$00)  ; |
EEPROM    ($40,$20,$20,$10,$20,$20,$40,$00)  ; }
EEPROM    ($40,$A8,$10,$00,$00,$00,$00,$00)  ; ~
EEPROM    ($70,$D8,$D8,$70,$00,$00,$00,$00)  ; DEL  = $7F
#endrem
